from __future__ import annotations

import datetime
import uuid

from decimal import Decimal
from typing import TYPE_CHECKING

from sqlalchemy import Date, ForeignKey, Numeric, UniqueConstraint, Uuid
from sqlalchemy.orm import Mapped, mapped_column, relationship

from ..db.base_class import reg


if TYPE_CHECKING:
    from .cost_centre import CostCentre
    from .stock_keeping_unit import StockKeepingUnit


@reg.mapped_as_dataclass(unsafe_hash=True)
class ClosingStock:
    __tablename__ = "closing_stocks"
    __table_args__ = (UniqueConstraint("date", "cost_centre_id", "sku_id"),)
    id: Mapped[uuid.UUID] = mapped_column(Uuid, primary_key=True, insert_default=uuid.uuid4)
    date_: Mapped[datetime.date] = mapped_column("date", Date, nullable=False)
    cost_centre_id: Mapped[uuid.UUID] = mapped_column(
        Uuid,
        ForeignKey("cost_centres.id"),
        nullable=False,
    )
    sku_id: Mapped[uuid.UUID] = mapped_column(Uuid, ForeignKey("stock_keeping_units.id"), nullable=False)
    quantity: Mapped[Decimal] = mapped_column(Numeric(precision=15, scale=2), nullable=False)

    cost_centre: Mapped["CostCentre"] = relationship("CostCentre")
    sku: Mapped["StockKeepingUnit"] = relationship("StockKeepingUnit")

    def __init__(
        self,
        date_: datetime.date,
        cost_centre_id: uuid.UUID,
        quantity: Decimal,
        sku_id: uuid.UUID | None = None,
        sku: "StockKeepingUnit" | None = None,
        id_: uuid.UUID | None = None,
    ):
        self.date_ = date_
        self.cost_centre_id = cost_centre_id
        self.quantity = quantity
        if sku_id is not None:
            self.sku_id = sku_id
        if sku is not None:
            self.sku = sku
            self.sku_id = sku.id
        if id_ is not None:
            self.id = id_
