import uuid

from typing import TYPE_CHECKING

from sqlalchemy import Boolean, Unicode, Uuid
from sqlalchemy.orm import Mapped, mapped_column, relationship

from ..db.base_class import reg
from ..schemas.cost_centre import CostCentreLink


if TYPE_CHECKING:
    # if the target of the relationship is in another module
    # that cannot normally be imported at runtime
    from .account import AccountBase
    from .journal import Journal


@reg.mapped_as_dataclass(unsafe_hash=True)
class CostCentre:
    __tablename__ = "cost_centres"

    id: Mapped[uuid.UUID] = mapped_column(Uuid, primary_key=True, insert_default=uuid.uuid4)
    name: Mapped[str] = mapped_column(Unicode, unique=True, nullable=False)
    is_fixture: Mapped[bool] = mapped_column(Boolean, nullable=False)

    accounts: Mapped[list["AccountBase"]] = relationship("AccountBase", back_populates="cost_centre")
    journals: Mapped[list["Journal"]] = relationship("Journal", back_populates="cost_centre")

    @property
    def __name__(self) -> str:
        return self.name

    def __init__(self, name: str, id_: uuid.UUID | None = None, is_fixture: bool = False) -> None:
        self.name = name
        if id_ is not None:
            self.id = id_
        self.is_fixture = is_fixture

    @classmethod
    def cost_centre_purchase(cls) -> uuid.UUID:
        return uuid.UUID("7b845f95-dfef-fa4a-897c-f0baf15284a3")

    @classmethod
    def cost_centre_kitchen(cls) -> uuid.UUID:
        return uuid.UUID("b2d398ce-e3cc-c542-9feb-5d7783e899df")

    @classmethod
    def cost_centre_overall(cls) -> uuid.UUID:
        return uuid.UUID("36f59436-522a-0746-ae94-e0f746bf6c0d")

    @classmethod
    def overall(cls) -> CostCentreLink:
        return CostCentreLink(id_=uuid.UUID("36f59436-522a-0746-ae94-e0f746bf6c0d"), name="Overall")
