from __future__ import annotations

import uuid

from datetime import date
from decimal import Decimal
from typing import TYPE_CHECKING

from sqlalchemy import Date, ForeignKey, Numeric, Text, UniqueConstraint, Uuid
from sqlalchemy.orm import Mapped, mapped_column, relationship

from ..db.base_class import reg
from .recipe_tag import RecipeTag


if TYPE_CHECKING:
    from .recipe_item import RecipeItem
    from .stock_keeping_unit import StockKeepingUnit
    from .tag import Tag


@reg.mapped_as_dataclass(unsafe_hash=True)
class Recipe:
    __tablename__ = "recipes"
    __table_args__ = (UniqueConstraint("sku_id", "date"),)

    id: Mapped[uuid.UUID] = mapped_column(Uuid, primary_key=True, insert_default=uuid.uuid4)
    sku_id: Mapped[uuid.UUID] = mapped_column(Uuid, ForeignKey("stock_keeping_units.id"), nullable=False)
    date_: Mapped[date] = mapped_column("date", Date, nullable=False, index=True)
    source: Mapped[str] = mapped_column(Text, nullable=False)
    instructions: Mapped[str] = mapped_column(Text, nullable=False)
    garnishing: Mapped[str] = mapped_column(Text, nullable=False)
    plating: Mapped[str] = mapped_column(Text, nullable=False)
    notes: Mapped[str] = mapped_column(Text, nullable=False)

    recipe_yield: Mapped[Decimal] = mapped_column(Numeric(precision=15, scale=2), nullable=False)

    items: Mapped[list["RecipeItem"]] = relationship("RecipeItem", back_populates="recipe")
    sku: Mapped["StockKeepingUnit"] = relationship("StockKeepingUnit", back_populates="recipes")
    tags: Mapped[list["Tag"]] = relationship(
        "Tag", secondary=RecipeTag.__table__, order_by="Tag.name", back_populates="recipes"
    )
