from __future__ import annotations

import uuid

from decimal import Decimal
from typing import TYPE_CHECKING

from sqlalchemy import ForeignKey, Numeric, Text, UniqueConstraint, Uuid
from sqlalchemy.orm import Mapped, mapped_column, relationship

from ..db.base_class import reg


if TYPE_CHECKING:
    from .product import Product
    from .recipe import Recipe


@reg.mapped_as_dataclass(unsafe_hash=True)
class RecipeItem:
    __tablename__ = "recipe_items"
    __table_args__ = (UniqueConstraint("recipe_id", "product_id"),)

    id: Mapped[uuid.UUID] = mapped_column(Uuid, primary_key=True, insert_default=uuid.uuid4)
    recipe_id: Mapped[uuid.UUID] = mapped_column(Uuid, ForeignKey("recipes.id"), nullable=False)
    product_id: Mapped[uuid.UUID] = mapped_column(Uuid, ForeignKey("products.id"), nullable=False)
    quantity: Mapped[Decimal] = mapped_column(Numeric(precision=15, scale=2), nullable=False)
    description: Mapped[str] = mapped_column(Text, nullable=False)

    recipe: Mapped["Recipe"] = relationship("Recipe", back_populates="items")
    product: Mapped["Product"] = relationship("Product")
