import uuid

from io import BytesIO

import brewman.schemas.image_upload as output

from fastapi import APIRouter
from fastapi.responses import StreamingResponse
from sqlalchemy import select
from sqlalchemy.orm import Session

from ..db.session import SessionFuture
from ..models.db_image import DbImage


router = APIRouter()


@router.get("/{id_}/{type_}", response_class=StreamingResponse)
def db_image(id_: uuid.UUID, type_: str) -> StreamingResponse:
    with SessionFuture() as db:
        item = db.execute(select(DbImage).where(DbImage.id == id_)).scalar_one()
        if type_ == "thumbnail":
            img = BytesIO(item.thumbnail)
        else:
            img = BytesIO(item.image)
        return StreamingResponse(img, media_type="image/jpeg")


def save_files(voucher_id: uuid.UUID, i: list[bytes], t: list[bytes], db: Session) -> None:
    i = i or []
    t = t or []
    for index, value in enumerate(i):
        db.add(DbImage(voucher_id, "voucher", i[index], t[index]))


def update_files(
    voucher_id: uuid.UUID,
    data: list[output.ImageUpload],
    i: list[bytes],
    t: list[bytes],
    db: Session,
) -> None:
    i = i or []
    t = t or []
    old = [f.id_ for f in data if f.id_]
    images = db.execute(select(DbImage).where(DbImage.resource_id == voucher_id)).scalars().all()
    for image in [i for i in images if i.id not in old]:
        db.delete(image)
    for index, value in enumerate(i):
        db.add(DbImage(voucher_id, "voucher", i[index], t[index]))
