import uuid

from datetime import date, datetime
from decimal import Decimal

from pydantic import (
    ConfigDict,
    FieldSerializationInfo,
    field_serializer,
    field_validator,
)
from pydantic.main import BaseModel

from . import to_camel
from .product import ProductLink


class ProductLedgerItem(BaseModel):
    id_: uuid.UUID | None = None
    date_: date
    name: str
    url: list[str]
    type_: str
    narration: str
    debit_quantity: Decimal | None = None
    debit_amount: Decimal | None = None
    debit_unit: str | None = None
    credit_quantity: Decimal | None = None
    credit_amount: Decimal | None = None
    credit_unit: str | None = None
    running_quantity: Decimal
    running_amount: Decimal
    posted: bool
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)

    @field_validator("date_", mode="before")
    @classmethod
    def parse_date(cls, value: date | str) -> date:
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()

    @field_serializer("date_")
    def serialize_date(self, value: date, info: FieldSerializationInfo) -> str:
        return value.strftime("%d-%b-%Y")


class ProductLedger(BaseModel):
    start_date: date | str
    finish_date: date | str
    product: ProductLink | None = None
    body: list[ProductLedgerItem]
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)

    @field_validator("start_date", mode="before")
    @classmethod
    def parse_start_date(cls, value: date | str) -> date:
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()

    @field_serializer("start_date")
    def serialize_start_date(self, value: date, info: FieldSerializationInfo) -> str:
        return value.strftime("%d-%b-%Y")

    @field_validator("finish_date", mode="before")
    @classmethod
    def parse_finish_date(cls, value: date | str) -> date:
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()

    @field_serializer("finish_date")
    def serialize_finish_date(self, value: date, info: FieldSerializationInfo) -> str:
        return value.strftime("%d-%b-%Y")
