import uuid

from datetime import date, datetime

from pydantic import (
    BaseModel,
    ConfigDict,
    FieldSerializationInfo,
    field_serializer,
    field_validator,
)

from . import to_camel


class LockDate(BaseModel):
    days: int | None = None
    date_: date | None = None
    model_config = ConfigDict(alias_generator=to_camel, populate_by_name=True)

    @field_validator("date_", mode="before")
    @classmethod
    def parse_date(cls, value: None | date | str) -> date | None:
        if value is None:
            return None
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()

    @field_serializer("date_")
    def serialize_date(self, value: date | None, info: FieldSerializationInfo) -> str | None:
        return None if value is None else value.strftime("%d-%b-%Y")


class VoucherTypesSelected(BaseModel):
    id_: int
    name: str
    model_config = ConfigDict(alias_generator=to_camel, str_strip_whitespace=True, populate_by_name=True)


class AccountTypesSelected(BaseModel):
    id_: int
    name: str
    model_config = ConfigDict(alias_generator=to_camel, populate_by_name=True)


class LockInformation(BaseModel):
    id_: uuid.UUID | None = None

    valid_from: date | None = None
    valid_till: date | None = None

    voucher_types: list[VoucherTypesSelected]
    account_types: list[AccountTypesSelected]

    start: LockDate
    finish: LockDate

    index: int
    model_config = ConfigDict(alias_generator=to_camel, populate_by_name=True)

    @field_validator("valid_from", mode="before")
    @classmethod
    def parse_valid_from(cls, value: date | str | None) -> date | None:
        if value is None:
            return None
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()

    @field_serializer("valid_from")
    def serialize_valid_from(self, value: date | None, info: FieldSerializationInfo) -> str | None:
        return None if value is None else value.strftime("%d-%b-%Y")

    @field_validator("valid_till", mode="before")
    @classmethod
    def parse_valid_till(cls, value: date | str | None) -> date | None:
        if value is None:
            return None
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()

    @field_serializer("valid_till")
    def serialize_valid_till(self, value: date | None, info: FieldSerializationInfo) -> str | None:
        return None if value is None else value.strftime("%d-%b-%Y")


class Maintenance(BaseModel):
    enabled: bool
    user: str


class AttendanceCount(BaseModel):
    attendance_count: int
    model_config = ConfigDict(alias_generator=to_camel, populate_by_name=True)
