from datetime import date, datetime
from decimal import Decimal

from pydantic import (
    ConfigDict,
    FieldSerializationInfo,
    field_serializer,
    field_validator,
)
from pydantic.main import BaseModel

from . import to_camel


class TrialBalanceItem(BaseModel):
    type_: str | None = None
    name: str | None = None
    debit: Decimal | None = None
    credit: Decimal | None = None
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)


class TrialBalance(BaseModel):
    date_: date | str
    body: list[TrialBalanceItem]
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)

    @field_validator("date_", mode="before")
    @classmethod
    def parse_date(cls, value: date | str) -> date:
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()

    @field_serializer("date_")
    def serialize_date(self, value: date, info: FieldSerializationInfo) -> str:
        return value.strftime("%d-%b-%Y")
