import { CommonModule } from '@angular/common';
import { NgModule, inject } from '@angular/core';
import { ActivatedRouteSnapshot, RouterModule, RouterStateSnapshot, Routes } from '@angular/router';

import { AuthGuard } from '../auth/auth-guard.service';

import { CostCentreDetailComponent } from './cost-centre-detail/cost-centre-detail.component';
import { CostCentreListComponent } from './cost-centre-list/cost-centre-list.component';
import { CostCentreListResolver } from './cost-centre-list-resolver.service';
import { CostCentreResolver } from './cost-centre-resolver.service';

const costCentreRoutes: Routes = [
  {
    path: '',
    component: CostCentreListComponent,
    canActivate: [
      (route: ActivatedRouteSnapshot, state: RouterStateSnapshot) =>
        inject(AuthGuard).canActivate(route, state),
    ],
    data: {
      permission: 'Cost Centres',
    },
    resolve: {
      list: () => inject(CostCentreListResolver).resolve(),
    },
  },
  {
    path: 'new',
    component: CostCentreDetailComponent,
    canActivate: [
      (route: ActivatedRouteSnapshot, state: RouterStateSnapshot) =>
        inject(AuthGuard).canActivate(route, state),
    ],
    data: {
      permission: 'Cost Centres',
    },
    resolve: {
      item: (route: ActivatedRouteSnapshot) => inject(CostCentreResolver).resolve(route),
    },
  },
  {
    path: ':id',
    component: CostCentreDetailComponent,
    canActivate: [
      (route: ActivatedRouteSnapshot, state: RouterStateSnapshot) =>
        inject(AuthGuard).canActivate(route, state),
    ],
    data: {
      permission: 'Cost Centres',
    },
    resolve: {
      item: (route: ActivatedRouteSnapshot) => inject(CostCentreResolver).resolve(route),
    },
  },
];

@NgModule({
  imports: [CommonModule, RouterModule.forChild(costCentreRoutes)],
  exports: [RouterModule],
  providers: [CostCentreListResolver, CostCentreResolver],
})
export class CostCentreRoutingModule {}
