import { HttpClient, HttpParams } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs/internal/Observable';
import { catchError } from 'rxjs/operators';

import { ErrorLoggerService } from '../core/error-logger.service';

import { RawMaterialCost } from './raw-material-cost';

const url = '/api/raw-material-cost';
const serviceName = 'RawMaterialCostService';

@Injectable({
  providedIn: 'root',
})
export class RawMaterialCostService {
  constructor(
    private http: HttpClient,
    private log: ErrorLoggerService,
  ) {}

  list(
    id: string | null,
    startDate: string | null,
    finishDate: string | null,
  ): Observable<RawMaterialCost> {
    const options = { params: new HttpParams() };
    if (startDate !== null) {
      options.params = options.params.set('s', startDate);
    }
    if (finishDate !== null) {
      options.params = options.params.set('f', finishDate);
    }
    let listUrl = '';
    if (id !== null) {
      listUrl = `${url}/${id}`;
    } else if (startDate || finishDate) {
      listUrl = `${url}/data`;
    } else {
      listUrl = url;
    }
    return this.http
      .get<RawMaterialCost>(listUrl, options)
      .pipe(catchError(this.log.handleError(serviceName, 'list'))) as Observable<RawMaterialCost>;
  }
}
