import { CommonModule } from '@angular/common';
import { NgModule, inject } from '@angular/core';
import { ActivatedRouteSnapshot, RouterModule, RouterStateSnapshot, Routes } from '@angular/router';

import { AuthGuard } from '../auth/auth-guard.service';

import { ReceiptAccountsResolver } from './receipt-accounts-resolver.service';
import { ReceiptResolver } from './receipt-resolver.service';
import { ReceiptComponent } from './receipt.component';

const receiptRoutes: Routes = [
  {
    path: '',
    component: ReceiptComponent,
    canActivate: [
      (route: ActivatedRouteSnapshot, state: RouterStateSnapshot) =>
        inject(AuthGuard).canActivate(route, state),
    ],
    data: {
      permission: 'journal',
    },
    resolve: {
      receiptAccounts: () => inject(ReceiptAccountsResolver).resolve(),
      voucher: (route: ActivatedRouteSnapshot) => inject(ReceiptResolver).resolve(route),
    },
    runGuardsAndResolvers: 'paramsChange',
  },
  {
    path: ':id',
    component: ReceiptComponent,
    canActivate: [
      (route: ActivatedRouteSnapshot, state: RouterStateSnapshot) =>
        inject(AuthGuard).canActivate(route, state),
    ],
    data: {
      permission: 'journal',
    },
    resolve: {
      receiptAccounts: () => inject(ReceiptAccountsResolver).resolve(),
      voucher: (route: ActivatedRouteSnapshot) => inject(ReceiptResolver).resolve(route),
    },
    runGuardsAndResolvers: 'paramsChange',
  },
];

@NgModule({
  imports: [CommonModule, RouterModule.forChild(receiptRoutes)],
  exports: [RouterModule],
  providers: [ReceiptResolver],
})
export class ReceiptRoutingModule {}
