import { CommonModule } from '@angular/common';
import { NgModule, inject } from '@angular/core';
import { ActivatedRouteSnapshot, RouterModule, RouterStateSnapshot, Routes } from '@angular/router';

import { AuthGuard } from '../auth/auth-guard.service';

import { RecipeTemplateDetailComponent } from './recipe-template-detail/recipe-template-detail.component';
import { RecipeTemplateListComponent } from './recipe-template-list/recipe-template-list.component';
import { RecipeTemplateListResolver } from './recipe-template-list-resolver.service';
import { RecipeTemplateResolver } from './recipe-template-resolver.service';

const recipeTemplateRoutes: Routes = [
  {
    path: '',
    component: RecipeTemplateListComponent,
    canActivate: [
      (route: ActivatedRouteSnapshot, state: RouterStateSnapshot) =>
        inject(AuthGuard).canActivate(route, state),
    ],
    data: {
      permission: 'Recipes',
    },
    resolve: {
      list: () => inject(RecipeTemplateListResolver).resolve(),
    },
  },
  {
    path: 'new',
    component: RecipeTemplateDetailComponent,
    canActivate: [
      (route: ActivatedRouteSnapshot, state: RouterStateSnapshot) =>
        inject(AuthGuard).canActivate(route, state),
    ],
    data: {
      permission: 'Recipes',
    },
    resolve: {
      item: (route: ActivatedRouteSnapshot) => inject(RecipeTemplateResolver).resolve(route),
    },
  },
  {
    path: ':id',
    component: RecipeTemplateDetailComponent,
    canActivate: [
      (route: ActivatedRouteSnapshot, state: RouterStateSnapshot) =>
        inject(AuthGuard).canActivate(route, state),
    ],
    data: {
      permission: 'Recipes',
    },
    resolve: {
      item: (route: ActivatedRouteSnapshot) => inject(RecipeTemplateResolver).resolve(route),
    },
  },
];

@NgModule({
  imports: [CommonModule, RouterModule.forChild(recipeTemplateRoutes)],
  exports: [RouterModule],
  providers: [RecipeTemplateListResolver, RecipeTemplateResolver],
})
export class RecipeTemplateRoutingModule {}
