import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs/internal/Observable';
import { catchError } from 'rxjs/operators';

import { ErrorLoggerService } from '../core/error-logger.service';

import { RecipeTemplate } from './recipe-template';

const url = '/api/recipe-templates';
const serviceName = 'RecipeTemplateService';

@Injectable({
  providedIn: 'root',
})
export class RecipeTemplateService {
  constructor(
    private http: HttpClient,
    private log: ErrorLoggerService,
  ) {}

  get(id: string | null): Observable<RecipeTemplate> {
    const getUrl: string = id === null ? url : `${url}/${id}`;
    return this.http
      .get<RecipeTemplate>(getUrl)
      .pipe(
        catchError(this.log.handleError(serviceName, `get id=${id}`)),
      ) as Observable<RecipeTemplate>;
  }

  list(): Observable<RecipeTemplate[]> {
    return this.http
      .get<RecipeTemplate[]>(`${url}/list`)
      .pipe(catchError(this.log.handleError(serviceName, 'list'))) as Observable<RecipeTemplate[]>;
  }

  save(recipeTemplate: RecipeTemplate): Observable<RecipeTemplate> {
    return this.http
      .post<RecipeTemplate>(`${url}`, recipeTemplate)
      .pipe(catchError(this.log.handleError(serviceName, 'save'))) as Observable<RecipeTemplate>;
  }

  update(recipeTemplate: RecipeTemplate): Observable<RecipeTemplate> {
    return this.http
      .put<RecipeTemplate>(`${url}/${recipeTemplate.id}`, recipeTemplate)
      .pipe(catchError(this.log.handleError(serviceName, 'update'))) as Observable<RecipeTemplate>;
  }

  saveOrUpdate(recipeTemplate: RecipeTemplate): Observable<RecipeTemplate> {
    if (!recipeTemplate.id) {
      return this.save(recipeTemplate);
    }
    return this.update(recipeTemplate);
  }

  delete(id: string): Observable<RecipeTemplate> {
    return this.http
      .delete<RecipeTemplate>(`${url}/${id}`)
      .pipe(catchError(this.log.handleError(serviceName, 'delete'))) as Observable<RecipeTemplate>;
  }
}
