import { CommonModule } from '@angular/common';
import { NgModule, inject } from '@angular/core';
import { ActivatedRouteSnapshot, RouterModule, RouterStateSnapshot, Routes } from '@angular/router';

import { AuthGuard } from '../auth/auth-guard.service';

import { RoleDetailComponent } from './role-detail/role-detail.component';
import { RoleListComponent } from './role-list/role-list.component';
import { RoleListResolver } from './role-list-resolver.service';
import { RoleResolver } from './role-resolver.service';

const roleRoutes: Routes = [
  {
    path: '',
    component: RoleListComponent,
    canActivate: [
      (route: ActivatedRouteSnapshot, state: RouterStateSnapshot) =>
        inject(AuthGuard).canActivate(route, state),
    ],
    data: {
      permission: 'Users',
    },
    resolve: {
      list: () => inject(RoleListResolver).resolve(),
    },
  },
  {
    path: 'new',
    component: RoleDetailComponent,
    canActivate: [
      (route: ActivatedRouteSnapshot, state: RouterStateSnapshot) =>
        inject(AuthGuard).canActivate(route, state),
    ],
    data: {
      permission: 'Users',
    },
    resolve: {
      item: (route: ActivatedRouteSnapshot) => inject(RoleResolver).resolve(route),
    },
  },
  {
    path: ':id',
    component: RoleDetailComponent,
    canActivate: [
      (route: ActivatedRouteSnapshot, state: RouterStateSnapshot) =>
        inject(AuthGuard).canActivate(route, state),
    ],
    data: {
      permission: 'Users',
    },
    resolve: {
      item: (route: ActivatedRouteSnapshot) => inject(RoleResolver).resolve(route),
    },
  },
];

@NgModule({
  imports: [CommonModule, RouterModule.forChild(roleRoutes)],
  exports: [RouterModule],
  providers: [RoleListResolver, RoleResolver],
})
export class RoleRoutingModule {}
