import { CommonModule } from '@angular/common';
import { NgModule, inject } from '@angular/core';
import { ActivatedRouteSnapshot, RouterModule, RouterStateSnapshot, Routes } from '@angular/router';

import { AuthGuard } from '../auth/auth-guard.service';

import { TrialBalanceResolver } from './trial-balance-resolver.service';
import { TrialBalanceComponent } from './trial-balance.component';

const trialBalanceRoutes: Routes = [
  {
    path: '',
    component: TrialBalanceComponent,
    canActivate: [
      (route: ActivatedRouteSnapshot, state: RouterStateSnapshot) =>
        inject(AuthGuard).canActivate(route, state),
    ],
    data: {
      permission: 'Trial Balance',
    },
    resolve: {
      info: (route: ActivatedRouteSnapshot) => inject(TrialBalanceResolver).resolve(route),
    },
    runGuardsAndResolvers: 'always',
  },
  {
    path: ':date',
    component: TrialBalanceComponent,
    canActivate: [
      (route: ActivatedRouteSnapshot, state: RouterStateSnapshot) =>
        inject(AuthGuard).canActivate(route, state),
    ],
    data: {
      permission: 'Trial Balance',
    },
    resolve: {
      info: (route: ActivatedRouteSnapshot) => inject(TrialBalanceResolver).resolve(route),
    },
    runGuardsAndResolvers: 'always',
  },
];

@NgModule({
  imports: [CommonModule, RouterModule.forChild(trialBalanceRoutes)],
  exports: [RouterModule],
  providers: [TrialBalanceResolver],
})
export class TrialBalanceRoutingModule {}
