import { CommonModule } from '@angular/common';
import { NgModule, inject } from '@angular/core';
import { ActivatedRouteSnapshot, RouterModule, RouterStateSnapshot, Routes } from '@angular/router';

import { AuthGuard } from '../auth/auth-guard.service';

import { UserDetailComponent } from './user-detail/user-detail.component';
import { UserListComponent } from './user-list/user-list.component';
import { UserListResolver } from './user-list-resolver.service';
import { UserResolver } from './user-resolver.service';

const userRoutes: Routes = [
  {
    path: '',
    component: UserListComponent,
    canActivate: [
      (route: ActivatedRouteSnapshot, state: RouterStateSnapshot) =>
        inject(AuthGuard).canActivate(route, state),
    ],
    data: {
      permission: 'Users',
    },
    resolve: {
      list: () => inject(UserListResolver).resolve(),
    },
  },
  {
    path: 'new',
    component: UserDetailComponent,
    canActivate: [
      (route: ActivatedRouteSnapshot, state: RouterStateSnapshot) =>
        inject(AuthGuard).canActivate(route, state),
    ],
    data: {
      permission: 'Users',
    },
    resolve: {
      item: (route: ActivatedRouteSnapshot) => inject(UserResolver).resolve(route),
    },
  },
  {
    path: ':id',
    component: UserDetailComponent,
    canActivate: [
      (route: ActivatedRouteSnapshot, state: RouterStateSnapshot) =>
        inject(AuthGuard).canActivate(route, state),
    ],
    resolve: {
      item: (route: ActivatedRouteSnapshot) => inject(UserResolver).resolve(route),
    },
  },
];

@NgModule({
  imports: [CommonModule, RouterModule.forChild(userRoutes)],
  exports: [RouterModule],
  providers: [UserListResolver, UserResolver],
})
export class UserRoutingModule {}
